# *****************************************************************************
#  Get_FileinfoAndOwner.ps1
#
#  Erstellt eine CSV-Datei (Get_FileinfoAndOwner.csv) mit den Namen,
#  Gren und Eigentmern aller Unterordner und Dateien, die in dem
#  als Parameter angegebenen Pfad enthalten sind.
#
#  HINWEIS: Fr den Zugriff auf die Informationen gelten die Rechte
#           des ausfhrenden Benutzers dieses Skripts. Hat er keine
#           Leserechte an einigen Objekten in der Pfadangabe, so
#           knnen diese nicht ausgewertet werden.
#
#  VERSION: 1.0a, 15.10.2015, Tobias Jacobi
#
# *****************************************************************************

# Skript-Parameter definieren // Define parameters of the script
Param
  (
  [string]$PV_Path
  )

# Pfad abfragen, sofern nicht als Parameter bergeben //
# Query the path if not given as parameter
If ($PV_Path.Trim() -like "")
  {
  Write-Host "Parameter PV_Path was not passed. Use /? to show some help text."
  [string]$GV_Path = Read-Host -Prompt "Enter the path (or /?)"
  }
Else
  {
  $GV_Path = $PV_Path
  }

# Verarbeitung starten, sofern ein Pfad angegeben wurde //
# Start evaluation if a path was given
If ($GV_Path.Trim() -notlike "")
  {
  # Falls der Hilfetext nicht angefordert wurde // If help text is not requested
  If ($GV_Path.Trim() -notlike "/?" )
    {
    # Logging aktivieren // Switch logging on
    Start-Transcript -Path .\Get_FileinfoAndOwner.log
    # Pfad rekursiv auswerten und Ergebnisse in die CSV-Datei schreiben //
	# Evaluate the path recursively and write the results to a CSV file
    Get-ChildItem $GV_Path -Force -Recurse | 
      Select-Object @{Name="Path";Expression={Split-Path -Parent $_.FullName}}, Name, LastWriteTime, @{Name="Kbytes";Expression={If($_.Mode -like "d*") {"<DIR>"} Else {"{0:N0}" -f ($_.Length / 1Kb)}}}, @{Name="Owner";Expression={(Get-Acl $_.FullName).Owner}} |
        Export-Csv -Path .\Get_FileinfoAndOwner.csv -Delimiter ";" -Encoding UTF8 -NoTypeInformation
    # Logging ausschalten // Switch logging off
    Stop-Transcript
    # Endmeldung ausgeben // Show end notice
    Write-Host "Files Get_FileinfoAndOwner.csv and Get_FileinfoAndOwner.log generated ..." -ForegroundColor green
	Start-Sleep -Seconds 5
    }
  Else
    {
	# Hilfetext ausgeben // Show the help text
	Write-Host "`r`nThe script creates a CSV file (Get_FileinfoAndOwner.csv) with the names,"
	Write-Host "sizes and owners of all subfolders and files in the specified path.`r`n"
	Write-Host "NOTE:       To access this information the script must have the rights to"
    Write-Host "            read them. If the executing user has not enough rights on some"
	Write-Host "            objects in the path, they can not be evaluated."
	Write-Host "            Check the generated logfile (Get_FileinfoAndOwner.log) to find"
	Write-Host "            possible errors.`r`n"
	Write-Host "SYNTAX:     .\Get_FileinfoAndOwner.ps1 [[-PV_Path] <String>]`r`n"
	Write-Host "EXAMPLE:    .\Get_FileinfoAndOwner.ps1 C:\TEMP`r`n"
	Write-Host "DISCLAIMER: This script is provided ""as is"" and without any warranty."
    Write-Host "            You use it at your own risk.`r`n"
	Read-Host -Prompt "Please press return to continue ..."
	}
  }
